/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package controller;

import cart.ShoppingCart;
import entity.Categories;
import entity.Features;
import entity.Orders;
import entity.Products;
import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;
import javax.ejb.EJB;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import session.CategoriesFacade;
import session.ClientsFacade;
import session.FeaturesFacade;
import session.GcceInformationFacade;
import session.OrdersFacade;
import session.ProductsFacade;

/**
 *
 * @author admin
 */
@WebServlet(name = "ControllerServlet",
            loadOnStartup = 1,
            urlPatterns = {"/category",
                           "/addToCart", 
                           "/viewCart",
                           "/updateCart",
                           "/checkout",
                           "/purchase",
                           "/chooseLanguage",
                           "/search",
                           "/login"})
public class ControllerServlet extends HttpServlet {

    private String surcharge;
    
    @EJB
    private CategoriesFacade categoryFacade;
    @EJB
    private GcceInformationFacade gcceInformationFacade;
    @EJB
    private ProductsFacade productFacade;
    @EJB
    private OrdersFacade ordersFacade;
    @EJB
    private FeaturesFacade featuresFacade;

    @Override
    public void init(ServletConfig servletConfig) throws ServletException {

        super.init(servletConfig);
        
        // initialize servlet with configuration information
        surcharge = servletConfig.getServletContext().getInitParameter("deliverySurcharge");
        
        // store variables in servlet context
        getServletContext().setAttribute("categories", categoryFacade.findAll());
        getServletContext().setAttribute("features", featuresFacade.findAll());
        getServletContext().setAttribute("currency", gcceInformationFacade.findByName("currency"));
        getServletContext().setAttribute("pagename", gcceInformationFacade.findByName("pagename"));
    }
    
    /**
     * Handles the HTTP <code>GET</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {

        String userPath = request.getServletPath();
        HttpSession session = request.getSession();
        Categories selectedCategory;
        Collection<Products> categoryProducts;
        
        // if category page is requested
        if (userPath.equals("/category")) {
            // get categoryId from request
            String categoryRoot = request.getQueryString();

            if (categoryRoot != null) {
                // get selected category
                selectedCategory = categoryFacade.findByRoot(categoryRoot);
                // place selected category in request scope
                session.setAttribute("selectedCategory", selectedCategory);
                // get all products for selected category
                categoryProducts = selectedCategory.getProductsCollection();
                // place category products in request scope
                session.setAttribute("categoryProducts", categoryProducts);
            }

        // if cart page is requested
        } else if (userPath.equals("/viewCart")) {
            
            String clear = request.getParameter("clear");

            if ((clear != null) && clear.equals("true")) {

                ShoppingCart cart = (ShoppingCart) session.getAttribute("cart");
                cart.clear();
            }

            userPath = "/cart";

        // if checkout page is requested
        } else if (userPath.equals("/checkout")) {
            
            ShoppingCart cart = (ShoppingCart) session.getAttribute("cart");

            // calculate total
            cart.calculateTotal(surcharge);

            // forward to checkout page and switch to a secure channel

        // if user switches language
        } else if (userPath.equals("/chooseLanguage")) {
            // TODO: Implement language request

            
        }
        
        

        // use RequestDispatcher to forward request internally
        String url = "/WEB-INF/view" + userPath + ".jsp";

        try {
            request.getRequestDispatcher(url).forward(request, response);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * Handles the HTTP <code>POST</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        String userPath = request.getServletPath();
        HttpSession session = request.getSession();
        ShoppingCart cart = (ShoppingCart) session.getAttribute("cart");
        Collection<Products> resultProducts;

        // if addToCart action is called
        if (userPath.equals("/addToCart")) {

            // if user is adding item to cart for first time
            // create cart object and attach it to user session
            if (cart == null) {

                cart = new ShoppingCart();
                session.setAttribute("cart", cart);
            }

            // get user input from request
            String productId = request.getParameter("productId");

            if (!productId.isEmpty()) {

                Products product = productFacade.find(Integer.parseInt(productId));
                cart.addItem(product);
            }

            userPath = "/category";


        // if updateCart action is called
        } else if (userPath.equals("/updateCart")) {

            // get input from request
            String productId = request.getParameter("productId");
            String quantity = request.getParameter("quantity");

            Products product = productFacade.find(Integer.parseInt(productId));
            cart.update(product, quantity);

            userPath = "/cart";


        // if purchase action is called
        } else if (userPath.equals("/purchase")) {
            if (cart != null) {

                // extract user data from request
                String username	= request.getParameter("username");
                String name = request.getParameter("name");
                String surname = request.getParameter("surname");
                String email = request.getParameter("email");
                String password = request.getParameter("password");
                String conf = request.getParameter("conf");
                String street = request.getParameter("street");
                String number = request.getParameter("streetnumber");
                String flat = request.getParameter("flat");
                String door = request.getParameter("door");
                String postcode	= request.getParameter("postcode");
                String city = request.getParameter("city");
                String province	= request.getParameter("province");
                
                Orders order = ordersFacade.placeOrder(username,name,surname,email,password,conf,street,number,flat,door,postcode,city,province,cart);
                session.setAttribute("order", order);
                cart.clear();
            }

            userPath = "/confirmation";

        
        // if purchase action is called
        } else if (userPath.equals("/login")) {
            String username	= request.getParameter("username");
            String password	= request.getParameter("pass");
            Orders order = ordersFacade.ordersFacade(username,password,cart);
            session.setAttribute("order", order);
            cart.clear();
                    
            userPath = "/confirmation";
        
        // If the user performed a search
        } else if (userPath.equals("/search")) {
            resultProducts = productFacade.findAll();
            Products product;
            Features f;
            int i;
            boolean valid;
            String id;
            String value;
            Iterator<Products> it = resultProducts.iterator();
            while(it.hasNext()) {
                product = it.next();
                valid = true;
                i = 1;
                id=request.getParameter(Integer.toString(i).concat("id"));
                while(id!=null&&valid) {
                    value = request.getParameter(Integer.toString(i));
                    if(!(value.equals("--")||value.equals(""))) {
                        f = featuresFacade.findById(id);
                        switch(f.getKindOfValue()) {
                            case 0: valid = product.isDiscreet(f, value);
                                break;
                            case 1: valid = product.checkHasInt(f, Integer.parseInt(value));
                                break;
                            case 2: valid = product.checkHasString(f, value);
                                break;
                            case 3: valid = product.checkHasDecimal(f, Float.parseFloat(value));
                        }
                    }
                    i++;
                    id=request.getParameter(Integer.toString(i).concat("id"));
                }
                if(!valid) {
                    resultProducts.remove(product);
                }
            }
            session.setAttribute("resultProducts", resultProducts);
            productFacade.flush();
            featuresFacade.flush();
            
            // forward to results page
            userPath = "/results";
        }
        
        
        // use RequestDispatcher to forward request internally
        String url = "/WEB-INF/view" + userPath + ".jsp";

        try {
            request.getRequestDispatcher(url).forward(request, response);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
}