/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package session;

import cart.ShoppingCart;
import cart.ShoppingCartItem;
import entity.Clients;
import entity.Contains;
import entity.ContainsPK;
import entity.Orders;
import entity.Products;
import entity.Statuses;
import java.math.BigDecimal;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import javax.annotation.Resource;
import javax.ejb.EJB;
import javax.ejb.SessionContext;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

/**
 *
 * @author admin
 */
@Stateless
@TransactionManagement(TransactionManagementType.CONTAINER)

public class OrdersFacade extends AbstractFacade<Orders> {
    @EJB
    private ClientsFacade clientsFacade;
    @PersistenceContext(unitName = "GCCEPU")
    private EntityManager em;
    @Resource
    private SessionContext context;

    @Override
    protected EntityManager getEntityManager() {
        return em;
    }

    public List<Orders> getPlacedOrders() {
        return em.createNamedQuery("Orders.findByStatus").setParameter("status",new Statuses(1)).getResultList();
    }
    
    public OrdersFacade() {
        super(Orders.class);
    }

    /**
     *
     * @param username
     * @param name
     * @param surname
     * @param email
     * @param password
     * @param conf
     * @param street
     * @param number
     * @param flat
     * @param door
     * @param postcode
     * @param city
     * @param province
     * @param cart
     * @return
     */
    @TransactionAttribute(TransactionAttributeType.REQUIRED)
    public Orders placeOrder(String username, String name, String surname, String email, String password, String conf, String street, String number, String flat, String door, String postcode, String city, String province, ShoppingCart cart) {
        try {
            if (!password.equals(conf)) {
                return null;
            }
            Clients client = addCustomer(username, name, surname, email, password, conf, street, number, flat, door, postcode, city, province);
            Orders order = addOrder(client, cart);
            addOrderedItems(order, cart);
            return order;
        } catch (Exception e){
            context.setRollbackOnly();
            return null;
        }
    }

    private Clients addCustomer(String username, String name, String surname, String email, String password, String conf, String street, String number, String flat, String door, String postcode, String city, String province) throws NoSuchAlgorithmException {
        Clients client = new Clients();
        client.setUsername(username);
        client.setFirstName(name);
        client.setSurname(surname);
        client.setEmail(email);
        MessageDigest md = MessageDigest.getInstance("SHA-256");
        md.update(password.getBytes());
        client.setPass(bytesToHex(md.digest()));
        client.setStreet(street);
        client.setStreetNumber(number);
        client.setFlat(Short.parseShort(flat));
        client.setDoor(Short.parseShort(door));
        client.setPostcode(postcode);
        client.setCity(city);
        client.setProvince(province);
        em.persist(client);
        return client;
    }

    private Orders addOrder(Clients client, ShoppingCart cart) {
        // set up customer order
    Orders order = new Orders();
    order.setClient(client);
    order.setOrderValue(BigDecimal.valueOf(cart.getTotal()));
    order.setStatus(new Statuses(1));
    order.setDateCreated(Calendar.getInstance().getTime());
    em.persist(order);
    return order;
    }

    private void addOrderedItems(Orders order, ShoppingCart cart) {
        em.flush();
        Iterator<ShoppingCartItem> it = cart.getItems().iterator();
        Collection<Contains> cc = new ArrayList();
        Contains c;
        Products p;
        ContainsPK cpk;
        ShoppingCartItem sc;
        while(it.hasNext()) {
            c = new Contains();
            cpk = new ContainsPK();
            sc = it.next();
            p = sc.getProduct();
            c.setOrders(order);
            c.setAmount(sc.getQuantity());
            c.setProducts(sc.getProduct());
            p.setQuantity(p.getQuantity()-sc.getQuantity());
            cpk.setOrderid(order.getIdOrder());
            cpk.setProductid(sc.getProduct().getIdProduct());
            c.setContainsPK(cpk);
            cc.add(c);
        }
        
        order.setContainsCollection(cc);
    }
    
    private String bytesToHex(byte[] bytes) {
        StringBuilder result = new StringBuilder();
        for (byte byt : bytes) {
            result.append(Integer.toString((byt & 0xff) + 0x100, 16).substring(1));
        }
        return result.toString();
    }

    public Orders ordersFacade(String username, String password, ShoppingCart cart) {
        try {
            MessageDigest md = MessageDigest.getInstance("SHA-256");
            md.update(password.getBytes());
            Clients client = clientsFacade.findByUsername(username);
            if(!bytesToHex(md.digest()).equals(client.getPass())) {
                return null;
            }
            Orders order = addOrder(client, cart);
            addOrderedItems(order, cart);
            return order;
        } catch (Exception e){
            context.setRollbackOnly();
            return null;
        }
    }
}
